// Vary a single tune over time using:
// Frequency
// Dynamics
// Timbre
// Texture
// Rhythm
// Harmony
// Spatialization
// Also have a mixture of elements at the same time using shreds

// Comb filter!!! Onto broad spectrum noise

// path
NRev reverb => dac;
0.025 => reverb.mix;
SndBuf rec => reverb;
SndBuf rec2 => reverb;
SndBuf rec3 => reverb;
SndBuf rec4 => reverb;

// the MIDI file in object
MidiFileIn min;
// the MIDI message shuttle
MidiMsg msg;

// the filename
string filename;
// if no command line arguments provided
if( me.args() == 0 ) me.dir() + "sonata.mid" => filename;
// else use that filename
else me.arg(0) => filename;

// open the file; exit on error
if( !min.open(filename) ) me.exit();

// // print out 
// cherr <= "----------" <= IO.newline();
// cherr <= "MIDI file: " <= IO.newline()
// <= " |- " <= filename <= IO.newline()
// <= " |- contains " <= min.numTracks() <= " tracks" <= // IO.newline();
// // print
// cherr <= "----------" <= IO.newline();
// cherr <= "playing..." <= IO.newline();
// cherr <= "----------" <= IO.newline();

// flag
int done;

fun int scale(int a, int sc[]) {
    sc.cap() => int n; //number of degrees in scale
    a/n => int o; //octave being requested, number of wraps
    a%n => a; //wrap the note within first octave
    
    if ( a<0 ) { //cover the negative border case
        a + 12 => a;
        o - 1 => o;
    }
    
    //each octave contributes 12 semitones, plus the scale
    return o*12 + sc[a];
}

fun void playNotes(Osc s)
{
    s.gain(.7);
    
    [0, 2, 4,  8, 10, 11] @=> int notes[];
    for(int i; i < notes.size(); i++)
    {   
        // set freq
        notes[i] + 57 => Std.mtof => s.freq;
        // go
        
        // advance time
        .2::second => now;
    }
    s.gain(0);
}

// entry point for each shred assigned to a track
fun void doTrack( int track, float speed )
{
    // hack polyphony
    // Wurley s[14];
    Bowed s[4]; 
    // BandedWG s[6];
    // BlowBotl s[14];
    
    // for each voice
    for( int i; i < s.size(); i++ )
    {
        // set gain
        1 => s[i].gain;
        // 1 => s[i].preset; // for BandedWG
        // connect to output
        s[i] => reverb;
        
        // for Bowed
        0.683858 => s[i].bowPressure;
        0.946597 => s[i].bowPosition;
        6.090591 => s[i].vibratoFreq;
        0.002753 => s[i].vibratoGain;
        0.694438 => s[i].volume;
    }
    // voice number for quick polyphony
    int v;
    
    // read through all MIDI messages on track
    while( min.read( msg, track ) )
    {
        // this means no more MIDI events at current time; advance time
        if( msg.when > 0::second )
            msg.when * speed => now; // speed of 1 is nominal
        
        // catch NOTEON messages (lower nibble == 0x90)
        if( (msg.data1 & 0xF0) == 0x90 && msg.data2 > 0 && msg.data3 > 0 )
        {
            // get the pitch and convert to frequencey; set
            msg.data2 => Std.mtof => s[v].freq;
            // velocity data; note on
            msg.data3/127.0 => s[v].noteOn;
            // cycle the voices
            (v+1)%s.size() => v;
            
            // log
            // cherr <= "NOTE ON track:" <= track <= " pitch:" <= msg.data2 <=" velocity:" <= msg.data3 <= IO.newline(); 
        }
    }
    
    // done with track
    done++;
}

// notes
20 => int fund;
[0, 1, 2,  3,  4,  5] @=> int oct[];
[0, 2, 4,  8, 10, 11] @=> int notes[];
.2 => float timi;

fun void noteSweep()
{
    BandedWG s[4];
    // BlowBotl s[14];
    
    // for each voice
    for( int i; i < s.size(); i++ )
    {
        // set gain
        1 => s[i].gain;
        1 => s[i].preset; // for BandedWG
        // connect to output
        s[i] => reverb;
    }
    // voice number for quick polyphony
    int v;
    
    for(int j; j < oct.size(); j++) 
    {
        for(int i; i < notes.size(); i++)
        {   
            // get the pitch and convert to frequencey; set
            Std.mtof(notes[i]+fund+12*oct[j]) => s[v].freq;
            // velocity data; note on
            40 => s[v].noteOn;
            // cycle the voices
            (v+1)%s.size() => v;
            
            timi::second => now;
        }
    }
    Std.mtof(notes[0]+fund+72) => s[v].freq;
    40 => s[v].noteOn;
    // 2::second => now;
    // 1 => s[v].noteOff;
}

fun void strings()
{
    // patch
    Bowed bow[5];
    
    // scale
    [0, 4, 7, 11, 14, 17] @=> int scale[];
    [0.0, 0.0, 1.0, .3, .3, 1.0 ] @=> float times[];
    
    // for each voice
    for( int i; i < bow.size(); i++ )
    {
        // set gain
        1 => bow[i].gain;
        // 2 => s[i].preset; // for BandedWG
        // connect to output
        bow[i] => reverb;
        
        0.683858 => bow[i].bowPressure;
        0.946597 => bow[i].bowPosition;
        6.090591 => bow[i].vibratoFreq;
        0.002753 => bow[i].vibratoGain;
        0.694438 => bow[i].volume;
    }
    
    int v;
    
    // infinite time loop
    for(int i; i < scale.size(); i++)
    {   
        // set freq
        scale[i] + 57 => Std.mtof => bow[v].freq;
        // go
        //.8 => bow.noteOn;
        .8 => bow[v].startBowing;
        // cycle the voices
        (v+1)%bow.size() => v;
        
        // advance time
        times[i]::second => now;
    }
    1 => bow[v].stopBowing;
    2::second => now;
    
    for( int i; i < bow.size(); i++ )
    {
        .1 => bow[i].stopBowing;
    }
}

// // delayed start
// 5::second => now;
// 
// 
// me.dir() + "whistle3.wav" => rec.read;
// 0 => rec.pos;
// 1 => rec.gain;
// 7::second => now;
// me.dir() + "whistle3.wav" => rec2.read;
// 1.5 => rec2.rate;
// 0 => rec.pos;
// 7::second => now;
// me.dir() + "whistle3.wav" => rec3.read;
// 0 => rec2.pos;
// 0 => rec.pos;
// 0.75 => rec3.rate;
// 1.5 => rec2.rate;
// 7::second => now;
// me.dir() + "whistle3.wav" => rec4.read;
// 0 => rec3.pos;
// 0 => rec2.pos;
// 0 => rec.pos;
// 1.25 => rec4.rate;
// 0.75 => rec3.rate;
// 1.5 => rec2.rate;
// 7::second => now;
// me.dir() + "whistle1.wav" => rec.read;
// 0 => rec.pos;
// 1 => rec.gain;
// 7::second => now;
// me.dir() + "whistle2.wav" => rec.read;
// me.dir() + "whistle2.wav" => rec2.read;
// me.dir() + "whistle2.wav" => rec3.read;
// me.dir() + "whistle2.wav" => rec4.read;
// 1.5 => rec4.rate;
// 0.625 => rec3.rate;
// 0.5 => rec2.rate;
// 7::second => now;


// // play buffers
// me.dir() + "whoosh.wav" => rec.read;
// 0 => rec.pos;
// 1 => rec.gain;
// 7::second => now;
// //"special:dope" => rec.read;
// //me.dir() + "snare.wav" => rec.read;
// 
// // play midi portion
// for( int t; t < min.numTracks(); t++ )
//     spork ~ doTrack( t, 1 );
// 35::second => now;
// // done
// cherr <= "done; cleaning up..." <= IO.newline();
// min.close();
// 
// strings();
// 
// // chaotic notes
// noteSweep();
// 
// 
// // sssss
// me.dir() + "sss.wav" => rec.read;
// 0 => rec.pos;
// 1 => rec.gain;
// // 6::second => now;
// 
// SinOsc s1 => reverb;
// playNotes(s1);


BlowBotl s => reverb;
BandedWG s0 => reverb;
SinOsc s1 => reverb;
TriOsc s2 => reverb;
SqrOsc s3 => reverb;
SawOsc s4 => reverb;
PulseOsc s5 => reverb;
Bowed bow => reverb;
        
0.683858 => bow.bowPressure;
0.946597 => bow.bowPosition;
6.090591 => bow.vibratoFreq;
0.002753 => bow.vibratoGain;
0.694438 => bow.volume;

fun void key1()
{
}





