// Visuals 
// ==============================================================================
// Visually, the entire sequencer is made up of "GMands" which are meant to read 
// Each mandala is a GPlane + mouse listener + state machine for handling input 
// and transitioning between various modes e.g. hovered, active, playing, etc.
// ==============================================================================

public class GCanv extends GGen {

    CustomGeometry customGeometry;
    MangoUVMaterial mangoMat;
    // GMesh mesh;
    // // set geo and material
    // mesh.set(customGeometry, mangoMat);
    // // add to scene
    // mesh --> this;

    vec3 positions[0];
    vec3 normals[0];
    vec2 uvs[0];
    vec3 colors[0];
    float height;
    float density;
    float radius;
    int numCirclePoints;
    int numPoints;
    float length;

    GPoints points --> this;

    fun void set( float h, float d, float r ) {
        h => height;
        d => density;
        r => radius;
        (height * density)$int * (density * 2 * Math.pi * radius)$int => numPoints;
        // set position
        // mesh.pos(@(0.0, -height/2.0, 0));
        points.pos(@(0.0, -height/2.0, 0)); // circle centered around 0
        
        // pass in 3D positions for each vertex
        for (int i; i < height * density; i++) {
            (2 * Math.pi * radius * density)$int => numCirclePoints;
            2 * Math.pi * radius => length;
            for (int j; j < numCirclePoints; j++) {
                -j/(1.0 * radius * density) => float theta;
                radius * Math.cos(theta) => float x;
                1.0 * i / density => float y;
                radius * Math.sin(theta) => float z;
                // calculate normal
                Math.cos(theta) => float nx;
                0 => float ny;
                Math.sin(theta) => float nz;
                positions << @(x, y, z);
                colors << @(1.0, 1.0, 1.0);
                normals << @(nx,ny,nz); // not calculated correctly
                uvs << @(j / numCirclePoints, i / height); // not calculated correctly
            }
        }
        points.geo().positions(positions);  // set vertex position data
        points.geo().colors(colors);   // set vertex color data
        customGeometry.positions(positions);

        // pass in the normals (used in lighting calculations), make sure they
        // are normalized (i.e. have magnitude = 1)
        customGeometry.normals(normals);

        // pass in the texture coordinates (used to map textures onto the mesh)
        // each field be clamped between 0 and 1
        customGeometry.uvs(uvs);
    }

    fun void color( vec3 c ) {
        for (int i; i < numPoints; i++) {
            c => colors[i];

        }
        points.geo().colors(colors);
    }

    fun float len() {
        return length;
    }

    fun vec3 getPos( float i ) {
        return positions[(i * density)$int]+this.pos();
    }

    fun vec3 getRot( float i ) {
        normals[(i * density)$int] => vec3 n;
        return @(0,Math.asin(-n.x),0);
    }

    fun void reverse( float speed ) {
        speed => float s;
        0 => float rotsum;
        0 => int polarity;
        while( polarity==0 || (polarity*rotsum) > -0.01 ) {
            this.rotate(@(0,-0.1*s*GG.dt(),0));
            -0.1*s*GG.dt() +=> rotsum;
            s+((-speed-s)*GG.dt()*4) => s;
            if( s*speed < 0 ) {
                if(s < 0)
                    -1 => polarity;
                else
                    1 => polarity;
            }
            // <<<polarity*rotsum, s, ((polarity*rotsum) > 0.05)>>>;
            GG.nextFrame() => now;
        }
    }
}