// Visuals 
// ==============================================================================
// Visually, the entire sequencer is made up of "GMands" which are meant to read 
// Each mandala is a GPlane + mouse listener + state machine for handling input 
// and transitioning between various modes e.g. hovered, active, playing, etc.
// ==============================================================================

public class GObs extends GGen {
    // initialize mesh
    .5 => float msize;
    GCube cube --> this;
    cube.scaWorld(@(msize,msize,msize));
    cube.mat().color(@(0,0,0));

    // initialize second hover circle
    // PlaneGeometry tg;
    // GLines plane --> this;
    // tg.set(1, 1, 5, 5);
    // plane.geo(tg);
    // plane.scaWorld(@(msize+.1,msize+.1,msize+.1));

    // reference to a mouse
    Mouse @ mouse;
    vec3 mousePos;

    // events
    Event onHoverEvent, onClickEvent;  // onExit, onRelease
    
    // states
    0 => static int NONE;  // not hovered or active
    1 => static int HOVERED;  // hovered
    2 => static int ACTIVE;   // clicked
    3 => static int PLAYING;  // makine sound!
    0 => int state; // current state
    0 => int lastState; // prev state

    // input types
    0 => static int MOUSE_HOVER;
    1 => static int MOUSE_EXIT;
    2 => static int MOUSE_CLICK;
    3 => static int NOTE_ON;
    4 => static int NOTE_OFF;

    // color map
    [
        Color.BLACK,    // NONE
        Color.RED,      // HOVERED
        Color.YELLOW,   // ACTIVE
        Color.GREEN     // PLAYING
    ] @=> vec3 colorMap[];

    NRev reverb => dac;
	BandedWG s => reverb;

	0.05 => float vol;
    70 => int notes;

    0 => s.gain; // set gain
    1 => s.preset; // for BandedWG

    // constructor
    fun void init(Mouse @ m, vec3 cc) {
        if (mouse != null) return;
        m @=> this.mouse;
    	// plane.mat().color(cc); // blend with canvas
    	cc => colorMap[NONE];
        spork ~ playAudio();
    }

    fun void colorCube(vec3 c) {
        cube.mat().color(c);
    }

    fun void colorRing(vec3 c) {
        c => colorMap[PLAYING];
    }

    // check if state is active (i.e. should play sound)
    fun int active() {
        return state == ACTIVE;
    }

    fun void play() {
        handleInput(NOTE_ON);
    }

    // stop play animation (called by sequencer on note off)
    fun void stop() {
        handleInput(NOTE_OFF);
    }

    // enter state, remember last state
    fun void enter(int s) {
        state => lastState;
        s => state;
    }

    // basic state machine for handling input
    fun void handleInput(int input) {
        if (input == NOTE_ON) {
            enter(PLAYING);
            return;
        }

        if (input == NOTE_OFF) {
            enter(lastState);
            return;
        }

        if (state == NONE) {
            if (input == MOUSE_HOVER)      enter(HOVERED);
            else if (input == MOUSE_CLICK) enter(ACTIVE);
        } else if (state == HOVERED) {
            if (input == MOUSE_EXIT)       enter(NONE);
            else if (input == MOUSE_CLICK) enter(ACTIVE);
        } else if (state == ACTIVE) {
            if (input == MOUSE_CLICK)      enter(NONE);
        } else if (state == PLAYING) {
            if (input == MOUSE_CLICK)      enter(NONE);
            if (input == NOTE_OFF)         enter(ACTIVE);
        }
    }

    // override ggen update
    fun void update(float dt) {

        // update state
        // plane.mat().color(colorMap[state]);
        // cube.mat().color( @(notes/100.0,(notes%10)/10.0,vol*2) );

    }

	fun void playAudio()
	{
	    while(true)
	    {
	        while (lastState != ACTIVE) {
	        	100::ms => now;
	        	Math.max(0,s.gain()-0.01) => s.gain;
	        }
	        while (state != PLAYING) {
	        	100::ms => now;
	        }
	        
            vol => s.gain;
            Std.mtof(notes) => s.freq;
            40 => s.noteOn;
            
            while (state == PLAYING) {
	        	100::ms => now;
	        }
	    }
	}

    fun void hop() {
        this.posWorld().y => float origy;
        0.5 => float vel;
        while(this.posWorld().y >= origy)
        {
            this.posWorld(this.posWorld() + @(0,vel,0));
            vel - 0.08 => vel;
            if(vel < 0) spork ~ flip();
            50::ms => now;
        }
    }

    fun void flip() {
        this.rot() => vec3 origrot;
        for(float i; i<2*Math.pi; 0.4 +=> i) {
            this.rotateOnWorldAxis( @(0,0,1), 0.4 );
            50::ms => now;
        }
        this.rot(origrot);
    }
}
