// Visuals 
// ==============================================================================
// Visually, the entire sequencer is made up of "GMands" which are meant to read 
// Each mandala is a GPlane + mouse listener + state machine for handling input 
// and transitioning between various modes e.g. hovered, active, playing, etc.
// ==============================================================================

public class GPlay extends GGen {
    // initialize mesh
    .5 => float msize;
    GSphere player --> this;
    player.scaWorld(@(msize,msize,msize));
    player.mat().color(@(0,0,0));
    float maxHeight;

    // [-1.0,-0.6,-0.2,0.2,0.6,1.0] @=> float segments[];
    [-1.0,-0.8,-0.6,-0.4,-0.2,0.0,0.2,0.4,0.6,0.8,1.0] @=> float segments[];
    [-9  ,-8  ,-5  ,-3  ,-1  ,0  ,2  ,5  ,7  ,9  ,11 ] @=> int scale[];
    ["data/hihat-open.wav","data/hihat-open.wav",
    "data/hihat.wav","data/hihat.wav",
    "data/kick.wav","data/kick.wav",
    "data/snare.wav","data/snare.wav",
    "data/snare-hop.wav","data/snare-hop.wav",
    "data/hihat-open.wav","data/hihat-open.wav",
    "data/kick.wav","data/kick.wav"] @=> string perc[];
    0 => int segmentNum;

    // reference to a mouse
    Mouse @ mouse;
    vec3 mousePos;

    // sounds
    BandedWG s => dac;
    0 => s.gain; // set gain
    1 => s.preset; // for BandedWG

    SndBuf buf[segments.size()];
    Gain g => dac;
    2 => g.gain;
    for(int i; i<segments.size();i++)
    {
        buf[i] => g;
        me.dir() + perc[i] => buf[i].read;
        buf[i].gain(0);
    }
    SndBuf hit => g;
    me.dir() + "data/kick.wav" => hit.read;
    0 => hit.gain;

    float notePos;
    float loopTime;
	0.05 => float vol;
    Math.random2(30,60) => int notes;
    

    // constructor
    fun void init(Mouse @ m, float mh) {
        mh => maxHeight;
        if (mouse != null) return;
        m @=> this.mouse;
        spork ~ set_segment();
    }

    fun void color(vec3 c) {
        player.mat().color(c);
    }

    fun void setScale(int start, int s[]) {
        start => notes;
        for (int i; i < scale.size(); i++) {
            if( i >= s.size() ) {
                s[i-s.size()]+12 => scale[i];
            } else {
                s[i] => scale[i];
            }
        }
    }

    fun void moveUp() {
        Math.min(player.posY() + .05, maxHeight) => player.posY;
    }

    fun void moveDown() {
        Math.max(player.posY() - .05, -maxHeight) => player.posY;
    }

    fun void set_segment()
    {
        while(true)
        {
            segmentNum => int lastSegment;
            for (int i; i < segments.size(); i++) {
                if (player.posY() < segments[i]*maxHeight) {
                    i => segmentNum;
                    break;
                }
            }
            if (lastSegment != segmentNum) {
                spork ~ play_background();
            }
            100::ms => now;
        }
    }

    fun void decay() {
        while (s.gain() > 0) 
        {
            Math.max(0,s.gain()-0.02) => s.gain;
            200::ms => now;
        }  
    }
    
    fun void play_background()
    {
        // play music with freq based on segment
        if (segmentNum%2 == 0) {
            // vol => s.gain;
            // Std.mtof(notes+scale[segmentNum]) => s.freq;
            // 40 => s.noteOn;
            // decay();
        } else {
            // play a percussive sound
            vol => buf[segmentNum].gain;
            0 => buf[segmentNum].pos;
        }
        vol => s.gain;
        Std.mtof(notes+scale[segmentNum]) => s.freq;
        40 => s.noteOn;
        decay();
    }

    fun void hitsound() {
        vol*2 => hit.gain;
        0 => hit.pos;
    }

    fun int checkCollision(GGen obstacles[]) {
        for( int i; i < obstacles.size(); i++ ) {
            if( collidesWith(obstacles[i]) ) {
                return i+1;
            }
        }
        return false;
    }

    fun int checkCollision(GGen object1, GGen objects2[]) {
        for( int i; i < objects2.size(); i++ ) {
            if( collidesWith(object1,objects2[i]) ) {
                return i+1;
            }
        }
        return false;
    }
    
    fun void reset() {
        player.posY(0);
    }

    fun int collidesWith(GGen o) {
        o.posWorld() => vec3 opos;
        o.scaWorld() => vec3 osca;
        player.posWorld() => vec3 ppos;
        player.scaWorld() => vec3 psca;
        if (ppos.x + psca.x/2 > opos.x - osca.x/4 + .05 &&
                ppos.x - psca.x/2 + .05 < opos.x + osca.x/4 &&
            ppos.y + psca.y/2 > opos.y - osca.y/4 + .05 &&
                ppos.y - psca.y/2 + .05 < opos.y + osca.y/4 &&
            ppos.z + psca.z/2 > opos.z - osca.z/4 + .05 &&
                ppos.z - psca.z/2 + .05 < opos.z + osca.z/4
            ) {
            // <<< opos, osca, ppos, psca >>>;
            return true;
        }
        return false;
    }

    fun int collidesWith(GGen o1, GGen o2) {
        o2.posWorld() => vec3 opos;
        o2.scaWorld() => vec3 osca;
        o1.posWorld() => vec3 ppos;
        o1.scaWorld() => vec3 psca;
        if (ppos.x + psca.x/2 > opos.x - osca.x/4 + .05 &&
                ppos.x - psca.x/2 + .05 < opos.x + osca.x/4 &&
            ppos.y + psca.y/2 > opos.y - osca.y/4 + .05 &&
                ppos.y - psca.y/2 + .05 < opos.y + osca.y/4 &&
            ppos.z + psca.z/2 > opos.z - osca.z/4 + .05 &&
                ppos.z - psca.z/2 + .05 < opos.z + osca.z/4
            ) {
            // <<< opos, osca, ppos, psca >>>;
            return true;
        }
        return false;
    }

    fun int xor(int a, int b) {
        return (a && !b) || (!a && b);
    }

    fun void squish( float speed, GGen obs ) {
        0 => float squishness;
        this.posWorld() => vec3 origpos;
        this.scaWorld() => vec3 origsca;
            origpos => vec3 ppos;
            origsca => vec3 psca;
        while( squishness<1 ) {
            obs.posWorld() => vec3 opos;
            obs.scaWorld() => vec3 osca;
            if(ppos.z + psca.z/2 < opos.z + osca.z/4 ) {
                // player is on the left
                ((opos.z - osca.z/4) - (ppos.z - psca.z/2))+0.05 => squishness;
                this.posWorld( origpos + @(0,0,-(origsca.z-squishness)/4));
            } else {
                // player is on the right
                ((ppos.z + psca.z/2) - (opos.z + osca.z/4))+0.05 => squishness;
                this.posWorld( origpos + @(0,0,(origsca.z-squishness)/4));
            }
            // <<< squishness >>>;
            
            this.scaWorld( @(origsca.x,origsca.y,squishness) );
            GG.nextFrame() => now;
        }
        this.scaWorld( origsca.x * @(1,1,1) );
        this.posWorld( origpos );
    }
}
