// uncomment to run in fullscreen
GG.fullscreen();

// Scene setup ===================
GG.scene().backgroundColor(@(.95,.8,.9));
GG.camera().pos( @(0,0,5) );
//GG.camera().rot( @(0,0,0) );
0 => float ss; // speed camera movement

// Create blank canvas
GPlane canvas --> GG.scene();
canvas.mat().color(@(1,1,1));
canvas.pos(@(0,0,0));
canvas.sca(@(13,2,1));

6 => int numMandala;
GCircle mandalas[numMandala];

FileTexture tex;
tex.path(me.dir() + "tex1.jpeg");


for(int i; i<numMandala; i++)
{
    mandalas[i] --> canvas;
    mandalas[i].pos(@(i/6.5-0.38,0,0.01));
    mandalas[i].scaWorld(@(1,1,1));
    mandalas[i].mat().color(@(0,0,0));

    PhongMaterial pmat;
    pmat.diffuseMap( tex );
    //mandalas[i].mat(pmat);
}

NRev reverb => dac;
[.05,.05,.05,.05,.05,.05] @=> float vol[];
[0,0,0,0,0,0] @=> int track[];
[70,70,70,70,70,70] @=> int notes[];

BandedWG s[numMandala];

// for each voice
for( int i; i < numMandala; i++ )
{
    // set gain
    vol[i] => s[i].gain;
    1 => s[i].preset; // for BandedWG
    // connect to output
    s[i] => reverb;
}



fun void move_camera() {

    if( KB.isKeyDown(KB.KEY_S) ) { // speed up
        if ( ss< 6 ) { ss+5*GG.dt() => ss; }
    } else { 1 => ss; }


    // 3d moving camera to figure out camera position
    if( KB.isKeyDown(KB.KEY_D) ) { // positive x axis
        GG.camera().translate(GG.dt()*ss*GG.camera().right());
    }
    if( KB.isKeyDown(KB.KEY_A) ) { // negative x axis
        GG.camera().translate(-GG.dt()*ss*GG.camera().right());
    }
    if( KB.isKeyDown(KB.KEY_E) ) { // positive y axis
        GG.camera().translate(GG.dt()*ss*GG.camera().forward());
    }
    if( KB.isKeyDown(KB.KEY_Z) ) { // negative y axis
        GG.camera().translate(-GG.dt()*ss*GG.camera().forward());
    }
    if( KB.isKeyDown(KB.KEY_W) ) { // positive z axis
        GG.camera().translate(GG.dt()*ss*GG.camera().up());
    }
    if( KB.isKeyDown(KB.KEY_X) ) { // negative z axis
        GG.camera().translate(-GG.dt()*ss*GG.camera().up());
    }
    if( KB.isKeyDown(KB.KEY_U) ) { // rotate + x axis
        GG.camera().rotateOnWorldAxis(GG.camera().right(), .3*GG.dt()*ss);
    }
    if( KB.isKeyDown(KB.KEY_M) ) { // rotate - x axis
        GG.camera().rotateOnWorldAxis(GG.camera().right(), -.3*GG.dt()*ss);
    }
    if( KB.isKeyDown(KB.KEY_I) ) { // rotate + y axis
        GG.camera().rotateOnWorldAxis(GG.camera().forward(), .3*GG.dt()*ss);
    }
    if( KB.isKeyDown(KB.KEY_N) ) { // rotate - y axis
        GG.camera().rotateOnWorldAxis(GG.camera().forward(), -.3*GG.dt()*ss);
    }
    if( KB.isKeyDown(KB.KEY_H) ) { // rotate + z axis
        GG.camera().rotateOnWorldAxis(GG.camera().up(), .3*GG.dt()*ss);
    }
    if( KB.isKeyDown(KB.KEY_K) ) { // rotate - z axis
        GG.camera().rotateOnWorldAxis(GG.camera().up(), -.3*GG.dt()*ss);
    }
    if( KB.isKeyDown(KB.KEY_P) ) { // print camera pos and angle
        <<< "Position: ", GG.camera().pos() >>>;
        <<< "Rotation: ", GG.camera().rot() >>>;
    }
}

fun void do_change(int i)
{
    if( KB.isKeyDown(KB.KEY_UP) ) {
        // increase vol
        vol[i] + .1 => vol[i];
    }
    if( KB.isKeyDown(KB.KEY_DOWN) ) {
        // decrease vol
        vol[i] - .1 => vol[i];
    }
    if( KB.isKeyDown(KB.KEY_LEFT) ) {
        // change track next
        notes[i] - 1 => notes[i];
    }
    if( KB.isKeyDown(KB.KEY_RIGHT) ) {
        // change track prev
        notes[i] + 1 => notes[i];
    }
}

fun void move_audio() {
    while(true) {
        if( KB.isKeyDown(KB.KEY_1) ) { // choose first circle
            do_change(0);

        }
        if( KB.isKeyDown(KB.KEY_2) ) { // choose first circle
            do_change(1);

        }
        if( KB.isKeyDown(KB.KEY_3) ) { // choose first circle
            do_change(2);

        }
        if( KB.isKeyDown(KB.KEY_4) ) { // choose first circle
            do_change(3);

        }
        if( KB.isKeyDown(KB.KEY_5) ) { // choose first circle
            do_change(4);

        }
        if( KB.isKeyDown(KB.KEY_6) ) { // choose first circle
            do_change(5);
        }
        100::ms => now;
    }
}

fun void audioLoop()
{
    while(true)
    {
        for(int i; i<numMandala; i++)
        {
            mandalas[i].mat().color( @(notes[i]/100.0,(notes[i]%10)/10.0,vol[i]) );
            vol[i] => s[i].gain;
            Std.mtof(notes[i]) => s[i].freq;
            40 => s[i].noteOn;
            200::ms => now;
        }
        
    }
}
spork ~ audioLoop();
spork ~ move_audio();

// infinite time loopp
while( true )
{
    move_camera();

    for(int i; i<numMandala; i++)
    {
        mandalas[i].rotateZ(GG.dt());
    }
    
    // IMPORTANT: synchronization point with next frame to render
    GG.nextFrame() => now;
}
